unit Datetime;
{This unit/library contains general functions and procedures for dealing
 with dates/times as required to support the DHCP Computerized Patient
 Record.  Please notify Curtis Anderson at the Salt Lake ISC of any
 errors encountered or changes/additions to this library.}

interface

{Functions exported:
                    SubDateTime        Subtracts time from current D/T.
                    FromFMDateTime     Converts DHCP FileMan Dates/Times to
                                       Delphi Dates/Times.
                    ToFMDateTime       Converts Delphi Dates/Times to
                                       DHCP FileMan Dates/Times.
                    WeekDateRange     Gets the week date range for a date.}

uses
SysUtils, WinTypes, WinProcs, Messages, Classes;

function ShortDateToFM(DelphiDT: TDateTime): string; {format windows short
  format date to FM}

function SubDateTime(SubDT: string)  {time increment to subract}
         : TDateTime;
{This function subtracts hours, days, months, years from currrent date/time
 based on the text in SubDT and returns the date/time as type TDateTime.}

function FromFMDateTime(FMDT: string; DoTime: bool) {FileMan date/time to convert}
         : TDateTime;
{This function converts a FileMan Date/Time (e.g. 2941222.172723 - 12/22/1994@17:27:23)
 passed as FMDT into a Delphi TDateTime (e.g. becomes .}

function ToFMDateTime(DelphiDT: TDateTime; DoTime: Bool) {Delphi date/time to convert}
         : string;
{This function converts a delphi Date/Time (e.g. )
 passed as DelphiDT into a FileMan Date/Time (e.g. becomes 2941222.172723)}

function WeekDateRange(DelphiDT: TDateTime) {Delphi D/T for calculations}
         : string;
{This function determines the begining and end dates for the week including the
 date passed in the format 'Delphi begin date^Delphi end date'}

 implementation

function SubDateTime(SubDT: string)  {time increment to subract}
         : TDateTime;
{This function subtracts hours, days, months, years from currrent date/time
 based on the text in SubDT and returns the date/time as type TDateTime.}
begin
    if SubDT = '4 hrs' then
       Result := Now - 0.166667
    else if SubDT = '8 hrs' then
        Result := Now - 0.333333
    else if SubDT = '12 hrs' then
        Result := Now - 0.5
    else if (SubDT = '1 day') or (SubDT = '24 hrs') then
        Result := Now - 1
    else if SubDT = '36 hrs' then
        Result := Now - 1.5
    else if (SubDT = '2 days') or (SubDT = '48 hrs') then
        Result := Now - 2
    else if (SubDT = '3 days') or (SubDT = '72 hrs') then
        Result := Now - 3
    else if SubDT = '7 days' then
        Result := Now - 7
    else if SubDT = '14 days' then
        Result := Now - 14
    else if SubDT = '30 days' then
        Result := Now - 30
    else if SubDT = '6 mos' then
        Result := Now - 180
    else if SubDT = '1 yr' then
        Result := Now - 365
    else if SubDT = '2 yrs' then
        Result := Now - 730
    else if SubDT = '5 yrs' then
        Result := Now - 1826
    else if SubDT = '10 yrs' then
        Result := Now - 3652
    else
        Result := 0;

end;   {end function SubDateTime}

function FromFMDateTime(FMDT: string; DoTime: bool) {FileMan date/time to convert}
         : TDateTime;
{This function converts a FileMan Date/Time (e.g. 2941222.172723 - 12/22/94@17:27:23)
 passed as FMDT into a Delphi TDateTime (e.g. becomes .}
var
   ce,yy,mm,dd,hh,nn,ss : string[2];
   DT : string[19];
begin
  if FMDT <> '' then
   begin
    ce := IntToStr(StrToInt(Copy(FMDT,1,1))+17) ;  {century from char 1}
    yy := Copy(FMDT,2,2);  {year from chars 2-3}
    mm := Copy(FMDT,4,2);  {month from chars 4-5}
    dd := Copy(FMDT,6,2);  {day from chars 6-7}
    if DoTime then
     begin
        hh := Copy(FMDT,9,2);  {hour from chars 9-10}
        nn := Copy(FMDT,11,2); {minute from chars 11-12}
        ss := Copy(FMDT,13,2); {second from chars 13-14}
     end;
    DT := mm + '/' + dd + '/' + ce + yy;
    if DoTime then DT := DT + ' ' + hh + ':' + nn + ':' + ss;
    Result := StrToDateTime(DT);
   end
  else
    Result := StrToDate('') ; 
end;   {end function FromFMDateTime}

function ToFMDateTime(DelphiDT: TDateTime; DoTime: bool) {Delphi date/time to convert}
         : string;
{This function converts a delphi Date/Time (e.g. )
 passed as DelphiDT into a FileMan Date/Time (e.g. becomes 2941222.172723)}
var
   ce,yy,mm,dd,hh,nn,ss : string[2];
   DT : string[19];
begin
  if DelphiDT > 0 then
   begin
    DT := FormatDateTime('mm/dd/yyyy hh:nn:ss', DelphiDT); {format returned: mm/dd/yyyy hh:nn:ss}
    ce := IntToStr(StrToInt(Copy(DT,7,2))-17) ;  {century from chars 7-8}
    mm := Copy(DT,1,2);  {month from chars 1-2}
    dd := Copy(DT,4,2);  {day from chars 4-5}
    yy := Copy(DT,9,2);  {year from chars 9-10}
    if DoTime then
     begin
        hh := Copy(DT,12,2); {hour from chars 12-13}
        nn := Copy(DT,15,2); {minute from chars 15-16}
        ss := Copy(DT,18,2); {second from chars 18-19}
     end;
    Result := ce + yy + mm + dd;
    if DoTime then Result := Result + '.' + hh + nn + ss;
   end
  else
    Result := '' ;
end;   {end function ToFMDateTime}

function ShortDateToFM(DelphiDT: TDateTime): string;
var
   ce,yy,mm,dd : string[2];
   DT: string[19];
begin
 if DelphiDT > 0 then
  begin
       DT := FormatDateTime('mm/dd/yyyy hh:nn:ss', DelphiDT); {format returned: mm/dd/yyyy hh:nn:ss}
       ce := IntToStr(StrToInt(Copy(DT,7,2))-17) ;  {century from chars 7-8}
       mm := Copy(DT,1,2);  {month from chars 1-2}
       dd := Copy(DT,4,2);  {day from chars 4-5}
       yy := Copy(DT,9,2);  {year from chars 9-10}
       Result := ce + yy + mm + dd ;
       {Result := '2' + yy + mm + dd;}
  end
 else
       Result := '' ; 
end;


function WeekDateRange(DelphiDT: TDateTime) {Delphi D/T for calculations}
         : string;
{This function determines the begining and end dates for the week including the
 date passed in the format: 'begin date (mm/dd/yy)^end date (mm/dd/yy)'}
var
   sdate,edate: TDateTime;
   dw,i : Integer;
begin
    dw := DayOfWeek(DelphiDT);
    i := 1;
    if dw = 1 then
        sdate := DelphiDT  {Sunday}
    else
        repeat    {Loop backwards until find Sunday}
            sdate := DelphiDT - i;
            dw := DayOfWeek(sdate);
            i := i + 1;
        until dw = 1;
    edate := sdate + 6;
    Result := DateToStr(sdate) + '^' + DateToStr(edate);
end;   {end function WeekDateRange}

end.
